#!/usr/bin/perl -w
use strict;

#########################

BEGIN { $| = 1; print "1..27\n"; }
my $loaded = 1;
END {print "not ok 1\n" unless $loaded;}
use MCS::CGI::HTML::Template;
use IO::File;

print "ok 1\n";

#########################

my $test = 1;

$test++;
my $p = new MCS::CGI::HTML::Template NAME => 'nico';
print $p ? "ok $test\n" : "not ok $test: Constructor failed\n";

$test++;
print $p->var('NAME') eq 'nico'
  ? "ok $test\n" : "not ok $test: \$p->var('NAME') = ". $p->var('NAME')." \n";

$test++;
$p->var(LASTNAME => 'simmons');
print $p->var('LASTNAME') eq 'simmons'
  ? "ok $test\n"
  : "not ok $test: \$p->var('LASTNAME') = $p->var('LASTNAME')\n";

open SAVEOUT, ">&STDOUT"
  or die "Cannot save STDOUT: $!\n";
my $test_file = 'test.out';
open STDOUT, "> $test_file"
    or die "Cannot redirect STDOUT to $test_file: $!\n";

# Declaration
$test++;
my $html = "<!DOCTYPE __NAME__>\n";
$p->parse($html);
my $text = `tail -1 $test_file`;
print SAVEOUT $text eq '<!DOCTYPE nico>'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Start tag
$test++;
$html = qq{<BODY BACKGROUND="__LASTNAME__">__\n};
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq '<BODY BACKGROUND="simmons">__'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# End tag
$test++;
$html = "</BODY __NAME__>\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq '</BODY nico>'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Text
$test++;
$html = "Text from __LASTNAME__ __LASTNAME\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq 'Text from simmons __LASTNAME'
  ? "ok $test\n" : "not ok $test: $text\n";

# A macro without a value
$test++;
$html = "Text from __FOOBAR__\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq 'Text from __FOOBAR__'
  ? "ok $test\n" : "not ok $test: $text\n";

# An innocent comment
$test++;
$html = "<!-- Innocent comment from __NAME__ -->\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq '<!-- Innocent comment from nico -->'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# END with neither IF nor WHILE is just an inncoent comment
$test++;
$html = "<!-- end FOOBAR -->\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq '<!-- end FOOBAR -->'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Syntax error: END without start
$test++;
$html = "<!-- end while -->\n";
print SAVEOUT "# Expect a syntax error\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq '<!-- end while -->'
  ? "ok $test\n"
  : "not ok $test: $text\n";

$test++;
$html = "<!-- end if -->\n";
print SAVEOUT "# Expect a syntax error\n";
$p->parse($html);
$text = `tail -1 $test_file`;
print SAVEOUT $text eq '<!-- end if -->'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# A conditional that will be true
$test++;
$p->var(TRUE => sub { 1 });
$p->parse("<!-- if TRUE -->");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "<!-- end if -->\n"
  ? "ok $test\n"
  : "not ok $test: $text\n";

$test++;
$p->parse("__NAME__<!-- end if -->\n");
$text = `tail -1 $test_file`;
#print SAVEOUT $text eq "<!-- if TRUE -->\n<!-- while FOOBAR -->\nnico"
print SAVEOUT $text eq "nico"
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Now a conditional that will be false
$test++;
$p->var(FALSE => sub { 0 });
print "\ntest $test";
$p->parse("<!-- if FALSE -->\n");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "test $test\n"
  ? "ok $test\n"
  : "not ok $test: $text\n";

$test++;
print "test $test\n";
$p->parse("__LASTNAME__<!-- end if -->\n");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "test $test\n"
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Check substitution within a conditional
$test++;
$p->parse("<!-- if TRUE --><A __LASTNAME__>foobar</A><!-- end if -->\n");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "<A simmons>foobar</A>"
  ? "ok $test\n"
  : "not ok $test: |$text|\n";

# Semantic error: IF without a CODEREF
$test++;
print SAVEOUT "# Expect an error\n";
$p->parse("<!-- if FOOBAR -->__LASTNAME__<!-- end if -->\n");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq 'simmons'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Set up a while loop with an array ref
$test++;
$p->var(COUNT => [{COUNT => 1, STRING => 'one'},
		  {COUNT => 2, STRING => 'two'},
		  {COUNT => 3, STRING => 'three'},
		 ]);
$p->parse("<!--while COUNT--><B>__COUNT__,__STRING__</B><!-- end while -->\n");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "<B>1,one</B><B>2,two</B><B>3,three</B>"
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Set up a while loop with a coderef
$test++;
my $count = 11;
$p->var(COUNT_SUB => sub {
	  $p->var(NUM => $count);
	  return $count++ < 14 ? 1 : 0;
	});
$p->parse("<!--while COUNT_SUB-->__NUM__%__NAME__\n<!-- end while -->\n");
$text = `tail -3 $test_file`;
print SAVEOUT $text eq "11%nico\n12%nico\n13%nico\n"
  ? "ok $test\n"
  : "not ok $test: $text\n";

# 0 iterations this time
$test++;
print "\ntest $test\n";
$count = 14;
$p->parse("<!--while COUNT_SUB-->__NUM__%__NAME__\n<!-- end while -->");
$text = `tail -2 $test_file`;
#print SAVEOUT $text eq "13%nico\n\n"
print SAVEOUT $text eq "test $test\n\n"
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Semantic error: WHILE with neither a coderef nor an array ref
$test++;
print SAVEOUT "# Expect an error\n";
$p->parse("<!-- while FOOBAR -->__LASTNAME__<!-- end while -->\n");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq 'simmons'
  ? "ok $test\n"
  : "not ok $test: $text\n";

# Nested if's and while's
$test++;
$p->parse("<!-- if TRUE -->__NAME__<!--if TRUE--> __LASTNAME__\n<!--end if--><!--end if-->");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "nico simmons\n"
  ? "ok $test\n"
  : "not ok $test: $text\n";

$test++;
$p->parse("<!--while COUNT--><!-- if TRUE -->foobar<!--end if--><!--end WHILE-->");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "foobar" x 3
  ? "ok $test\n"
  : "not ok $test: $text\n";

$test++;
print "test $test\n";
$p->parse("<!-- if TRUE --><!--while COUNT-->foobar<!--end WHILE--><!--end if-->");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "foobar" x 3
  ? "ok $test\n"
  : "not ok $test: $text\n";

$test++;
print "test $test\n";
$count = 11;
$p->parse("<!-- while COUNT_SUB --><!--while COUNT-->foobar<!--end WHILE--><!--end while-->");
$text = `tail -1 $test_file`;
print SAVEOUT $text eq "foobar" x 9
  ? "ok $test\n"
  : "not ok $test: $text\n";

__END__
END {
    unlink $test_file
 	or warn "Cannot unlink $test_file: $!\n";
}
